<?php

namespace App\Http\Controllers\Admin\CMS;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ResponseCodeEnum;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\CMS\SettingRepository;
use App\Repositories\Validators\CMS\SettingValidator;

/**
 * Class SettingsController.
 *
 * @package namespace App\Http\Controllers\CMS;
 */
class SettingsController extends Controller
{
    /**
     * @var SettingRepository
     */
    protected $repository;

    /**
     * @var SettingValidator
     */
    protected $validator;

    /**
     * SettingsController constructor.
     *
     * @param SettingRepository $repository
     * @param SettingValidator $validator
     */
    public function __construct(SettingRepository $repository, SettingValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    public function index()
    {
        $courses = $this->repository->paginate(request('per_page', self::PAGE_NUM));
        return Response::success($courses);
    }


    public function store(Request $request)
    {
        $this->validate($request, [
            'name' => 'required',
            'key' => ['required', Rule::unique('cms_settings')->whereNull('deleted_at')],
            'type' => 'required|integer',
            'body' => 'nullable',
            'sort' => 'required|integer',
            'status' => 'required|integer',
        ]);
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));

            $data['key'] = strtoupper($data['key']);
            $model = $this->repository->create($data);
            return Response::success($model);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }

    public function show($id)
    {
        $model = $this->repository->find($id);
        return Response::success($model);
    }

    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name' => 'required',
            'key' => ['required', Rule::unique('cms_settings')->whereNull('deleted_at')->ignore($id)],
            'type' => 'required|integer',
            'body' => 'nullable',
            'sort' => 'required|integer',
            'status' => 'required|integer',
        ]);
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
            $data['key'] = strtoupper($data['key']);
            $model = $this->repository->update($data, $id);
            return Response::success($model);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    public function destroy($id)
    {
        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }
}
