<?php

namespace App\Http\Controllers\Api\Note;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Note\Comment;
use App\Repositories\Models\Note\Note;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Note\CommentRepository;
use App\Repositories\Validators\Note\CommentValidator;

/**
 * Class CommentsController.
 *
 * @package namespace App\Http\Controllers\Note;
 */
class CommentsController extends Controller
{
    /**
     * @var CommentRepository
     */
    protected $repository;

    /**
     * @var CommentValidator
     */
    protected $validator;

    /**
     * CommentsController constructor.
     *
     * @param CommentRepository $repository
     * @param CommentValidator $validator
     */
    public function __construct(CommentRepository $repository, CommentValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    public function index(Request $request)
    {
        $lists = $this->repository->paginate(request('per_page', self::PAGE_NUM));

        return Response::success($lists);
    }


    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
            $data['user_id'] = login_user_id();
            $model = $this->repository->create($data);

            Note::query()->where('id', $model['note_id'])->increment('comment_count');
            return Response::success($model);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }


    public function show($id)
    {
        $model = $this->repository->find($id);
        return Response::success($model);
    }


    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
            $model = $this->repository->update($data, $id);
            return Response::success($model);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }


    public function destroy($id)
    {
        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function good(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $id = $request->get('id');

        $user_id = login_user_id();

        $cache = Cache::get("cache:comment:good:{$id}:{$user_id}");
        if ($cache) {
            return Response::fail(T("Do not repeat operation."), ResponseCodeEnum::SYSTEM_ERROR);
        }

        Comment::query()->where('id', $id)->increment('good_count');

        Cache::put("cache:comment:good:{$id}:{$user_id}", 1, Carbon::now()->addHours(2));
        return Response::success(null);
    }

    public function answer(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $id = $request->get('id');

        $comment = Comment::query()->where('parent_id', 0)->where('id', $id)->first();
        $comment->is_answer = 1;
        $comment->save();

        Note::query()->where('id', $comment->note_id)->update(['is_solve' => 1]);

        return Response::success(null);
    }
}
